from navconfig import config


SLACK_APP_ID = config.get('SLACK_APP_ID')
SLACK_CLIENT_ID = config.get('SLACK_CLIENT_ID')
SLACK_CLIENT_SECRET = config.get('SLACK_CLIENT_SECRET')
SLACK_SIGNING_SECRET = config.get('SLACK_SIGNING_SECRET')
# Bot information:
SLACK_BOT_TOKEN = config.get('SLACK_BOT_TOKEN')
SLACK_DEFAULT_CHANNEL = config.get('SLACK_DEFAULT_CHANNEL')
SLACK_TEAM_ID = config.get('SLACK_TEAM_ID')
