from navconfig import config

# Office 365
O365_CLIENT_ID = config.get('O365_CLIENT_ID')
O365_CLIENT_SECRET = config.get('O365_CLIENT_SECRET')
O365_TENANT_ID = config.get('O365_TENANT_ID')
O365_USER = config.get('O365_USER')
O365_PASSWORD = config.get('O365_PASSWORD')
