"""Notify Meta information."""

__title__ = 'async-notify'
__description__ = ('Library for send notifications. '
                   'simple and powerful asyncio-based library for sending notifications.')
__version__ = '0.8.11'
__author__ = 'Jesus Lara'
__author_email__ = 'jesuslara@phenobarbital.info'
__license__ = 'APACHE'
